package xyz.pixelatedw.finalbeta;

import java.util.ArrayList;
import java.util.List;

import com.electronwill.nightconfig.core.file.CommentedFileConfig;
import com.electronwill.nightconfig.core.file.CommentedFileConfigBuilder;
import com.google.common.base.Strings;

public class ModConfig {

	private static final String CONFIG_PATH = "config/finalbeta.toml";
	private static final List<Option> OPTIONS = new ArrayList<>();

	private CommentedFileConfig fileConfig;

	public static final Option<Boolean> SUGAR_CANE_ON_SAND = make("Sugar Cane on sand", true, "Allows sugar canes to be placed on sand");
	public static final Option<Boolean> ADD_MORE_SOUNDS = make("Add more sounds", true,
			"Links a few more sounds from your local 'resources' folder with the game, namely for item breaking, minecarts and chests");
	public static final Option<Boolean> ENABLE_CLOUDS = make("Enable Clouds", true, "Enables the rendering of clouds");
	public static final Option<Boolean> DISABLE_ID_TAGS = make("Disable ID Tags", true,
			"Disables id tags showing above entities in F3 mode");
	public static final Option<Boolean> FENCE_SLIM_HITBOX = make("Slim Hitbox for Fences", true,
			"Uses slim hitbox for fences on the X and Z axies (the height of the fence is not changed)");
	public static final Option<Boolean> FENCE_CONNECT = make("Fence connections", true,
			"Visually connects fences with nearby solid blocks");
	public static final Option<Boolean> REMOVE_NIGHTMARES = make("Remove Nightmares", false,
			"Removes nightmares completely");
	public static final Option<Boolean> DISABLE_BEDS = make("Disable Beds", false,
			"Disables bed mechanics such as setting spawn points and skipping the night, does NOT delete already placed blocks nor does it disable the crafting or placing of beds as decorative blocks");
	public static final Option<Integer> BOAT_BREAKING_LOGIC = make("Boat Breaking Logic", 2,
			"0 - Vanilla behavior\n1 - Boats will never break regardless of their speed\n2 - Boats will only break when crashing with almost max speed");
	public static final Option<Boolean> STACK_DROP = make("Drop Held Stack", true,
			"Allows the player to drop the entire stack they're holding using Shift + Q (or whatever drop key they have set)");
	public static final Option<Boolean> ENABLE_TIME_TRACKING = make("Enable Time Tracking", true,
			"Adds a Days Played entry in the F3 overlay displaying number of ingame days and real life days played in that world");
	public static final Option<Boolean> ENABLE_REPAIR = make("Enable Item Repair", true,
			"Allows 2 or more items of the same type to be repaired using a crafting table using the same formula as modern day vanilla uses");
	public static final Option<Boolean> ENABLE_WHITE_WOOL_RECIPE = make("Enable White Wool Recipe", true,
			"Allows dyed wool to be dyed back white using bone meal");
	public static final Option<Boolean> ENABLE_GHASTS_INSTA_DEATH = make("Enable Ghast Insta Death", true,
			"Allows ghasts to die in 1 shot when their fireballs get reflected at them");
	public static final Option<Double> MINECART_MAX_SPEED = make("Minecart maximum speed limit", 0.5,
			"Maximum allowed speed for minecarts\nVanilla default is 0.4");
	public static final Option<Double> MINECART_POWERED_BOOST = make("Powered rails speed boost", 1.15,
			"Extra speed modifier when minecarts go over powered rails\nVanilla default is 1.0");
	public static final Option<Boolean> FURNACE_MINECART_PUSH = make("Enable furnace minecarts push", true,
			"Enables furance minecarts to push forward the first minecart it comes in contact with");
	public static final Option<Boolean> FURNACE_MINECART_CHUNK_LOADING = make("Enable furnace minecarts to load chunks", true,
			"Name says it all, furnace minecarts will load chunks as they go on tracks.");
	public static final Option<Double> CLOUDS_HEIGHT = make("Clouds Height", 0.0,
			"Clouds height modifier, goes from 0.0, meaning the default 108 blocks height, to 1.0, meaning a 324 blocks height.");
	public static final Option<Boolean> ENABLE_COAL_AND_REDSTONE_BLOCKS_RECIPE = make("Enable Coal and Redstone Blocks Recipe", false,
			"Allows crafting coal and redstone as blocks for better storage");
	public static final Option<Boolean> DISABLE_EATING_WHEN_MAX_HP = make("Disable eating when at max HP", false,
			"Makes it so players can no longer eat if they're at max HP so they don't accidently waste food");
	public static final Option<Double> APPLE_DROP_RATE = make("Apple Drop Rate", 0.0,
			"Chance for apples to drop from leaves (decimal between 0.0 and 1.0)");
	public static final Option<Boolean> EDIT_SIGNS = make("Edit Signs", true, "Enables sign editing");

	public static final Option<Boolean> FIX_BOW_MODEL = make("Fix bow model", true,
			"Makes the box model held by players and skeletons bigger and facing forward");
	public static final Option<Boolean> FIX_MINECART_FLICKERING = make("Fix minecart flickering", true,
			"Fixes minecarts flickering when looking at their backs as a passanger");
	public static final Option<Boolean> FIX_MINECART_STOPPING_ON_ITEMS = make("Fix minecart stopping on items", true,
			"Fixes minecarts getting stopped by arrows and dropped items on tracks");
	public static final Option<Boolean> FIX_FISHING = make("Fix fishing", true, "Fixes fishes going above the player's head when fishing");
	public static final Option<Boolean> FIX_LEG_ARMOR_ON_VEHICLES = make("Fix leg armor on vehicles", true,
			"Fixes leg armor not properly getting updated when switching poses (start/stop riding a vehicle)");
	public static final Option<Boolean> FIX_STAIRS_DROPS = make("Fix stairs drops", true, "Fix stairs not dropping themselves when mined");
	public static final Option<Boolean> FIX_PICKAXE_EFFECTIVENESS = make("Fix pickaxe effectiveness", true,
			"Fixes pickaxes not being effective agaist certain blocks that it should be effective on");
	public static final Option<Boolean> FIX_AXE_EFFECTIVENESS = make("Fix axe effectiveness", true,
			"Fixes axes not being effective agaist certain blocks that it should be effective on");
	public static final Option<Boolean> FIX_SHOVEL_EFFECTIVENESS = make("Fix shovel effectiveness", true,
			"Fixes shovels not being effective agaist certain blocks that it should be effective on");
	public static final Option<Boolean> FIX_SADDLES_NOT_DROPPING = make("Fix saddles not dropping", true,
			"Fixes saddles not dropping when killing saddled pigs");
	public static final Option<Boolean> FIX_FURNACE_LAVA_BUCKET = make("Fix furnace lava bucket", true,
			"Fixes furnaces consuming the bucket when using lava buckets as fuel");
	public static final Option<Boolean> FIX_BOOKSHELVES_DROP = make("Fix Bookshelves Drops", true,
			"Drops 3 books when breaking a bookshelf");
	public static final Option<Boolean> FIX_DOUBLE_DOORS = make("Fix Double Doors", true,
			"Fixes double doors not being in their correct state when pressure plates are used to open them");
	public static final Option<Boolean> FIX_SLABS_RECIPE = make("Fix Slabs Recipe", true,
			"Givs two slabs for every block, since 1 block = 2 slabs => 3 blocks = 6 slabs (instead of 3)");
	public static final Option<Boolean> FIX_SLIME_SPLITS = make("Fix Slime Splits", true,
			"Fixes slimes not splitting if their HP is not exactly zero");
	public static final Option<Boolean> FIX_BOW_DURABILITY = make("Bow Durability", true,
			"Enables durability stat on bows (385 uses)");
	
	private static ModConfig instance = new ModConfig();
	public static final ModConfig instance() {
		return instance;
	}

	public ModConfig() {
		CommentedFileConfigBuilder builder = CommentedFileConfig.builder(CONFIG_PATH);
		this.fileConfig = builder.autosave().autoreload().build();

		this.fileConfig.load();

		for (Option o : OPTIONS) {
			if (!this.fileConfig.contains(o.name)) {
				this.fileConfig.add(o.name, o.defaultValue);
				this.fileConfig.setComment(o.name, o.description);
			} else if (!Strings.isNullOrEmpty(o.description) && this.fileConfig.contains(o.name)
					&& !this.fileConfig.containsComment(o.name)) {
				this.fileConfig.setComment(o.name, o.description);
			}
		}
	}

	public static <T> Option<T> make(String name, T defaultValue, String description) {
		Option<T> option = new Option(name, defaultValue, description);
		OPTIONS.add(option);
		return option;
	}

	public <T> T get(Option<T> o) {
		return this.fileConfig.getOrElse(o.name, o.defaultValue);
	}

	public <T> T set(Option<T> o, T value) {
		return this.fileConfig.set(o.name, value);
	}

	public static class Option<T> {
		private String name;
		private String description;
		private T defaultValue;

		public Option(String name, T defaultValue, String description) {
			this.name = name;
			this.defaultValue = defaultValue;
			this.description = description;
		}

		public Option(String name, T defaultValue) {
			this.name = name;
			this.defaultValue = defaultValue;
		}

		public T get() {
			return ModConfig.instance().get(this);
		}

		public void set(T value) {
			ModConfig.instance().set(this, value);
		}
	}
}
