package xyz.pixelatedw.finalbeta.mixin;

import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.At.Shift;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;

import net.minecraft.entity.Boat;
import net.minecraft.util.maths.MathsHelper;
import xyz.pixelatedw.finalbeta.ModConfig;
import xyz.pixelatedw.finalbeta.WyHelper;

@Mixin(Boat.class)
public class BoatMixin {

	@Inject(method = "tick", at = @At(value = "INVOKE", target = "Ljava/lang/Math;sqrt(D)D", shift = Shift.AFTER))
	public void boatVelocityCheck(CallbackInfo ci) {
		Boat boat = ((Boat) (Object) this);

		/*
		 * This randomly named field_1624 is actually the horizontal collision
		 * check, which we're setting to false before the vanilla check happens
		 * if the boat is set to survive the impact from our checks.
		 * 
		 * Do note that the way the boat collision check happens is that the
		 * boat gets slowed down first, and then the velocity check is
		 * performed, by default a velocity check is 0.15, we're raising that
		 * limit to 0.22.
		 */
		if (this.isCollidingHorizontally(boat) && this.canSurvive(boat)) {
			((Boat) (Object) this).field_1624 = false;
		}
		
		if (boat.passenger != null && boat.field_1645 % 39 == 1) {
			float speed = MathsHelper.sqrt(boat.velocityX * boat.velocityX + boat.velocityZ * boat.velocityZ);
			float volume = 0.3f + WyHelper.lerp(WyHelper.clamp(speed, 0.0F, 0.25F), 0.0F, 0.7f);
			boat.level.playSound(boat.x, boat.y, boat.z, "sound3.liquid.water", volume, 0.0f);
		}
	}

	private boolean canSurvive(Boat boat) {
		double velocity = Math.sqrt(boat.velocityX * boat.velocityX + boat.velocityZ * boat.velocityZ);
		boolean survives = false;
		if (ModConfig.BOAT_BREAKING_LOGIC.get() == 1) {
			survives = true;
		} else if (ModConfig.BOAT_BREAKING_LOGIC.get() == 2 && velocity < 0.22) {
			survives = true;
		}

		return survives;
	}

	private boolean isCollidingHorizontally(Boat boat) {
		return ((Boat) (Object) this).field_1624;
	}
}
