package xyz.pixelatedw.finalbeta.mixin;

import org.spongepowered.asm.mixin.Mixin;
import org.spongepowered.asm.mixin.injection.At;
import org.spongepowered.asm.mixin.injection.Inject;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfo;
import org.spongepowered.asm.mixin.injection.callback.CallbackInfoReturnable;

import net.minecraft.entity.Entity;
import net.minecraft.entity.LivingEntity;
import net.minecraft.entity.animal.Chicken;
import net.minecraft.entity.animal.Pig;
import net.minecraft.entity.monster.Ghast;
import net.minecraft.entity.monster.Spider;
import net.minecraft.entity.player.Player;
import net.minecraft.entity.projectile.Snowball;
import net.minecraft.item.ItemType;
import xyz.pixelatedw.finalbeta.ModConfig;
import xyz.pixelatedw.finalbeta.WyHelper;

@Mixin(LivingEntity.class)
public class LivingEntityMixin {

	@Inject(method = "setSize", at = @At("HEAD"), cancellable = true)
	public void setSize(float f, float f1, CallbackInfo ci) {
		LivingEntity entity = (LivingEntity) (Object) this;
		if (entity instanceof Chicken) {
			entity.width = 0.3F;
			entity.height = 0.7F;
			ci.cancel();
		}
	}

	@Inject(method = "dropLoot", at = @At("TAIL"))
	public void dropLoot(CallbackInfo ci) {
		if (ModConfig.FIX_SADDLES_NOT_DROPPING.get()) {
			LivingEntity entity = (LivingEntity) (Object) this;
			if (entity instanceof Pig && ((Pig) entity).isSaddled()) {
				entity.dropItem(ItemType.saddle.id, 1);
			}
		}
	}
	
	@Inject(method = "damage", at = @At("HEAD"), cancellable = true)
	public void damage(Entity thrower, int damage, CallbackInfoReturnable<Boolean> cir) {
		LivingEntity entity = (LivingEntity) (Object) this;
		if (ModConfig.ENABLE_GHASTS_INSTA_DEATH.get() && thrower != null && entity instanceof Ghast && thrower instanceof Snowball) {
			LivingEntity fireballThrower = ((Snowball) thrower).field_2193;
			if (fireballThrower instanceof Player) {
				entity.damage(fireballThrower, 1_000);
				cir.setReturnValue(true);
			}
		}
	}
	
	@Inject(method = "travel", at = @At("HEAD"))
	public void travel(float f, float f1, CallbackInfo ci) {
		LivingEntity entity = (LivingEntity) (Object) this;
		if (ModConfig.ADD_MORE_SOUNDS.get() && entity.isOnLadder() && !(entity instanceof Spider)
				&& (entity.velocityY > 0.1 || entity.velocityY < -0.1) && entity.field_1645 % 15 == 0 && !entity.onGround) {
			float pitch = WyHelper.clamp(0.85f + entity.level.rand.nextFloat() / 2, 0.0f, 1.0f);
			entity.level.playSound(entity.x, entity.y, entity.z, "sound3.step.ladder", 0.3f, pitch);
		}
	}
}
